#!/bin/bash

function usage {
 echo -e "Usage: \"./genCycle <n>\", n >= 3" 
}

function genDomain {
 NUM_NODES=$1
 echo "(define (domain uts_cycle_$NUM_NODES)"
 echo " (:requirements :typing :conditional-effects)"
 echo " (:types node label)"
 constants="(:constants "
 for ((i=0;i< NUM_NODES ;i++)); do
  constants="$constants n$i"
 done
 constants=$constants" - node)"
 echo " $constants"
 echo " (:predicates"
 echo "	(visited ?n - node)"
 echo "	(edge-label ?n1 ?n2 - node ?l - label)"
 echo "	(at-node ?n - node)"
 echo "  )"
 echo "  (:action follow-label"
 echo "   :parameters (?label - label)"
 echo "   :precondition (and)"
 echo "   :effect "
 echo "   (and "
 for((s=0; s < NUM_NODES; s++)); do {
  if [ $s -eq 0 ] ; then
   NEXT=`expr $s + 1`
   PREV=`expr $NUM_NODES - 1`
  elif [ $s -eq `expr $NUM_NODES - 1` ] ; then
   NEXT=0
   PREV=`expr $s - 1`
  else
   NEXT=`expr $s + 1`
   PREV=`expr $s - 1`
  fi
  echo "    (when (and (at-node n$s) (edge-label n$s n$NEXT ?label))"
  echo "      (and  (at-node n$NEXT) (not (at-node n$s)) (visited n$NEXT)))"
  echo "    (when (and (at-node n$s) (edge-label n$s n$PREV ?label))"
  echo "      (and  (at-node n$PREV) (not (at-node n$s)) (visited n$PREV)))"
 }; done
 echo "   ))"
 echo " )"
}

function genProblem {
 NUM_NODES=$1
 echo "(define (problem uts_cycle_$NUM_NODES)"
 echo " (:domain uts_cycle_$NUM_NODES)"
 echo " (:objects l1 l2 - label)"
 echo " (:init"
 echo "  (oneof"
 for((i=0;i<NUM_NODES;i++)); do
  echo "   (and (at-node n$i) (visited n$i))"
 done
 echo "  )"
 for((i=0;i<NUM_NODES;i++)); do
  if [ $i -eq 0 ] ; then
   NEXT=`expr $i + 1`
   PREV=`expr $NUM_NODES - 1`
  elif [ $i -eq `expr $NUM_NODES - 1` ] ; then
   NEXT=0
   PREV=`expr $i - 1`
  else
   NEXT=`expr $i + 1`
   PREV=`expr $i - 1`
  fi
  echo "  (oneof"
  echo "   (and (edge-label n$i n$NEXT l1) (edge-label n$i n$PREV l2))"
  echo "   (and (edge-label n$i n$NEXT l2) (edge-label n$i n$PREV l1))"
  echo "  )"  
 done
 echo "  )"
 echo " (:goal (and"
 for((i=0;i<NUM_NODES;i++)); do
  echo "        (visited n$i)"
 done
 echo "       ))"
 echo ")"
}

if [ -z $1 ]; then 
 usage
else
 NUM_NODES=$1
 echo "Generating $NUM_NODES node UTS cycle"
 genDomain $NUM_NODES > dc"$NUM_NODES".pddl
 genProblem $NUM_NODES > pc"$NUM_NODES".pddl
fi

