(define (domain forest)
  (:types
     ;;;;;;;top level grid;;;;;;;;
     location
     problem

     ;;;;;;;subproblem;;;;;;;
     sub-location 
     log-location
     package
     truck
     plane
     city
     block
  )
  (:constants
     x1 y1 x2 y2 - location

     ;;;;;;;sub-problem types;;;;;
     grid logistics blocksworld - problem

     ;;;;;;;blocksworld subproblem;;;
     a b c - block

     ;;;;;;;logistics subproblem;;;
     t1 t2 - truck
     a1 - plane
     p1 - package
     l11 l12 l21 l22 - log-location
     c1 c2 - city
  )
  (:predicates 
    ;;grid coordinates
   (at ?loc - location)
   (succ-loc ?loc1 ?loc2 - location)

    ;;subproblem at grid point is solved
   (solved ?loc - location)

   ;;subproblem type at each grid point
   (problem-at ?loc - location ?p - problem)

   ;;the subproblem at grid point can be solved
   (enabled ?loc - location) 
    ;;used to link subproblems
   (enables ?loc ?loc1 - location)

   ;;;;;grid navigation subproblem;;;;;;;;;;;;;
   (s-at-x ?loc - sub-location ?l - location)
   (s-at-y ?loc - sub-location ?l - location)
   (s-succ-loc ?loc1 ?loc2 - sub-location)
   (s-goal-x ?loc - sub-location ?l - location)
   (s-goal-y ?loc - sub-location ?l - location)

   ;;;;;logistics subproblem;;;;;;;;;;;;;;;;;;;
   (s-pkg-at ?p - package ?loc - log-location ?l - location)
   (s-truck-at ?t - truck ?loc - log-location ?l - location)
   (s-plane-at ?t - plane ?loc - log-location ?l - location)
   (s-city-loc ?loc - log-location ?c - city)
   (s-airport-loc ?loc - log-location)
   (s-pkg-in-t ?p - package ?t - truck ?l - location)
   (s-pkg-in-a ?p - package ?a - plane ?l - location)

   ;;;;;blocksworld subproblem;;;;;;;;;;;;;;;;;
   (s-on ?b1 ?b2 - block ?l - location)
   (s-on-table ?b - block ?l - location)
   (s-clear ?b - block ?l - location)
   (s-empty ?l - location)
   (s-holding ?b - block ?l - location)
  )

  ;;;;;top-level grid navigation actions;;;;;;;
  (:action up-x
   :effect (and
	(when (and (at x1) (at y1) (solved x1)) 
              (and (not (at x1)) (at x2)))
	(when (and (at x1) (at y2) (solved x1))
              (and (not (at x1)) (at x2)))
  ))

  (:action up-y
   :effect (and
	(when (and (at x1) (at y1) (solved y1))
              (and (not (at y1)) (at y2)))
	(when (and (at x2) (at y1) (solved y1)) 
              (and (not (at y1)) (at y2)))
 ))
  (:action down-x
   :effect (and
	(when (and (at x2) (at y1) (solved x2)) 
              (and (not (at x2)) (at x1)))
	(when (and (at x2) (at y2) (solved x2)) 
              (and (not (at x2)) (at x1)))
  ))

  (:action down-y
  :effect (and
	(when (and (at x1) (at y2) (solved y2)) 
              (and (not (at y2)) (at y1)))
	(when (and (at x2) (at y2) (solved y2)) 
              (and (not (at y2)) (at y1)))
 ))

 (:action enable_subproblem
  :parameters (?loc ?loc1 - location)
  :precondition (and)
  :effect (when (and (solved ?loc) (enables ?loc ?loc1)) (enabled ?loc1))
 )

  ;;;;;grid navigation sub-problem;;;;;;;;;;;

  (:action s-up-x
   :parameters (?sloc1x ?sloc1y ?sloc2x - sub-location
                ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc) 
			(s-succ-loc ?sloc1x ?sloc2x)	
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-x ?sloc2x ?loc) (not (s-at-x ?sloc1x ?loc))))
  )

  (:action s-up-y
   :parameters (?sloc1x ?sloc1y ?sloc2y - sub-location
                ?loc - location)
   :precondition (and) 
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
			(s-succ-loc ?sloc1y ?sloc2y)
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-y ?sloc2y ?loc) (not (s-at-y ?sloc1y ?loc))))
  )

  (:action s-down-x
   :parameters (?sloc1x ?sloc1y ?sloc2x - sub-location
               ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
			 (s-succ-loc ?sloc2x ?sloc1x)
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-x ?sloc2x ?loc) (not (s-at-x ?sloc1x ?loc))))
  )

  (:action s-down-y
   :parameters (?sloc1x ?sloc1y ?sloc2y - sub-location
                ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
			(s-succ-loc ?sloc2y ?sloc1y)
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-y ?sloc2y ?loc) (not (s-at-y ?sloc1y ?loc))))
  )

  (:action solve-grid
   :parameters (?slocx ?slocy - sub-location
	        ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
		      (s-at-x ?slocx ?loc) (s-at-y ?slocy ?loc)
                     (s-goal-x ?slocx ?loc) (s-goal-y ?slocy ?loc)
                 )
		(solved ?loc) 
           )
  )

  ;;;;;logistics sub-problem;;;;;;;;;;;

  (:action load-truck
   :parameters (?t - truck ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
    	 	      (s-truck-at ?t ?loc ?l) (s-pkg-at ?p ?loc ?l))
                 (and (s-pkg-in-t ?p ?t ?l) (not (s-pkg-at ?p ?loc ?l))))
  )

  (:action load-plane
   :parameters (?a - plane ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-plane-at ?a ?loc ?l) (s-pkg-at ?p ?loc ?l))
		(and (s-pkg-in-a ?p ?a ?l) (not (s-pkg-at ?p ?loc ?l))))
  )

  (:action unload-truck
   :parameters (?t - truck ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-truck-at ?t ?loc ?l) (s-pkg-in-t ?p ?t ?l))
		(and (s-pkg-at ?p ?loc ?l) (not (s-pkg-in-t ?p ?t ?l))))
  )

  (:action unload-plane
   :parameters (?a - plane ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-plane-at ?a ?loc ?l) (s-pkg-in-a ?p ?a ?l))
		(and (s-pkg-at ?p ?loc ?l) (not (s-pkg-in-a ?p ?a ?l))))
  )

  (:action move-truck
   :parameters (?t - truck ?loc1 ?loc2 - log-location ?c - city ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-truck-at ?t ?loc1 ?l) (s-city-loc ?loc1 ?c)
                      (s-city-loc ?loc2 ?c) (not (= ?loc1 ?loc2))
                 )
		(and (not (s-truck-at ?t ?loc1 ?l)) (s-truck-at ?t ?loc2 ?l)))
  ) 

  (:action move-plane
   :parameters (?a - plane ?loc1 ?loc2 - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-plane-at ?a ?loc1 ?l) (s-airport-loc ?loc1)
                      (s-airport-loc ?loc2) (not (= ?loc1 ?loc2))
                 )
		(and (not (s-plane-at ?a ?loc1 ?l)) (s-plane-at ?a ?loc2 ?l)))
  ) 

  (:action solve-logistics
   :parameters (?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
                      (s-pkg-at p1 l22 ?l)
                 )
	        (solved ?l) 
           )
  )

  ;;;;;blocksworld sub-problem;;;;;;;;;;;
  (:action stack
   :parameters (?b1 ?b2 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
		       (s-holding ?b1 ?loc) (s-clear ?b2 ?loc))
	 	(and (s-on ?b1 ?b2 ?loc) (not (s-holding ?b1 ?loc)) 
                     (not (s-clear ?b2 ?loc)) (s-clear ?b1 ?loc) (s-empty ?loc)))
  )

  (:action putdown
   :parameters (?b1 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-holding ?b1 ?loc))
		(and (s-on-table ?b1 ?loc) (not (s-holding ?b1 ?loc)) 
                     (s-clear ?b1 ?loc) (s-empty ?loc)))
 )

  (:action unstack
   :parameters (?b1 ?b2 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-empty ?loc) (s-clear ?b1 ?loc) (s-on ?b1 ?b2 ?loc))
		(and (not (s-on ?b1 ?b2 ?loc)) (s-holding ?b1 ?loc) (s-clear ?b2 ?loc)
                     (not (s-clear ?b1 ?loc)) (not (s-empty ?loc))))
  )
  (:action pickup
   :parameters (?b1 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-empty ?loc) (s-clear ?b1 ?loc)
                 )
		(and (not (s-on-table ?b1 ?loc)) (s-holding ?b1 ?loc) 
                     (not (s-clear ?b1 ?loc)) (not (s-empty ?loc))))
  )

  (:action solve-blocksworld
   :parameters (?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-on a b ?loc) (s-on b c ?loc)
                 )
		(solved ?loc) 
           )
  )

)
