#!/bin/bash

function usage {
 echo -e "Usage: \"./genFullConnected <n>\", n >= 4" 
}

function genDomain {
 NUM_NODES=$1
 echo "(define (domain uts_full_connected_$NUM_NODES)"
 echo " (:requirements :typing :conditional-effects)"
 echo " (:types node label)"
 constants="(:constants "
 for ((i=0;i< NUM_NODES ;i++)); do
  constants="$constants n$i"
 done
 constants=$constants" - node)"
 echo " $constants"
 echo " (:predicates"
 echo "	(visited ?n - node)"
 echo "	(edge-label ?n1 ?n2 - node ?l - label)"
 echo "	(at-node ?n - node)"
 echo "  )"
 echo "  (:action follow-label"
 echo "   :parameters (?label - label)"
 echo "   :precondition (and)"
 echo "   :effect "
 echo "   (and "
 for((s=0; s < NUM_NODES; s++)); do {
  for((t=0; t < NUM_NODES; t++)); do {
  if [ $s -eq $t ] ; then
    NULL=0 #do nothing
  else
   echo "    (when (and (at-node n$s) (edge-label n$s n$t ?label))"
   echo "      (and  (at-node n$t) (not (at-node n$s)) (visited n$t)))"
  fi
  }; done
 }; done
 echo "   ))"
 echo " )"
}

NARRAY[0]=0
NARRAY[1]=0
NARRAY[2]=0

LARRAY[0]=0
LARRAY[1]=0
LARRAY[2]=0


function recurseLabeling {
 NODE=$1
 NUM_NODES=$2
 ASSIGNED=$3
 NUM_LABELS=`expr $NUM_NODES - 1`
# echo recurse $1 $2 $3
 if [ $ASSIGNED -eq  $NUM_LABELS ]; then
#   echo TERM
   LINE=" (and"
   for((t=0; t < $NUM_LABELS; t++)); do {
     LINE="$LINE (edge-label n$NODE n${NARRAY[ $t ]} l${LARRAY[ $t ]})"
   }; done
   LINE="$LINE )\n"
   echo -e $LINE
 else
#     echo REC $ASSIGNED
  OUT=""
  for((j=0; j < $NUM_LABELS; j++)); do {
#     echo j = $j
   LABEL_USED=0
   for((i=0; i < $ASSIGNED; i++)); do {
     EQ=`expr  ${LARRAY[$i]} = $j`
     #echo LARRAY[$i] = ${LARRAY[$i]}, j = $j, EQ = $EQ
     if [ $EQ -eq 1 ] ; then
      LABEL_USED=1
     fi
   }; done
   NEXT_NODE=0
   #echo USED = $LABEL_USED
   if [ $LABEL_USED -eq 0 ]; then
    LARRAY[$ASSIGNED]=$j
#     echo set LARRAY[$ASSIGNED] = ${LARRAY[$ASSIGNED]}
    ASSN=`expr $ASSIGNED + 1`
    OUT="$OUT `recurseLabeling $NODE $NUM_NODES $ASSN`"
   fi
  }; done
  echo -e $OUT
 fi
 
}

function makeLabeling {
  NUM_NODES=$2
  NODE=$1
  ASSIGNED=0

  ARRAY_INDX=0
  for((i=0; i < NUM_NODES; i++)); do {
   if [ $i -ne $NODE ]; then
    NARRAY[$ARRAY_INDX]=$i
#    echo set NARRAY[$ARRAY_INDX] = ${NARRAY[$ARRAY_INDX]}
    ARRAY_INDX=`expr $ARRAY_INDX + 1`
   fi
  }; done

  echo -e `recurseLabeling $NODE $NUM_NODES $ASSIGNED`

}

function genProblem {
 NUM_NODES=$1
 echo "(define (problem uts_full_connected_$NUM_NODES)"
 echo " (:domain uts_full_connected_$NUM_NODES)"
 objects="(:objects "
 for ((i=0;i< NUM_NODES-1 ;i++)); do
  objects="$objects l$i"
 done
 objects=$objects" - label)"
 echo $objects
 echo " (:init"
 echo "  (oneof"
 for((i=0;i<NUM_NODES;i++)); do
  echo "   (and (at-node n$i) (visited n$i))"
 done
 echo "  )"

 for((s=0; s < NUM_NODES; s++)); do {
  echo "  (oneof"
  echo -e `makeLabeling $s $NUM_NODES`
  echo "  )"
 }; done

 echo "  )"
 echo " (:goal (and"
 for((i=0;i<NUM_NODES;i++)); do
  echo "        (visited n$i)"
 done
 echo "       ))"
 echo ")"
}

if [ -z $1 ]; then 
 usage
else
 NUM_NODES=$1
 echo "Generating $NUM_NODES node UTS cycle"
 genDomain $NUM_NODES > dfc"$NUM_NODES".pddl
 genProblem $NUM_NODES > pfc"$NUM_NODES".pddl
fi

