(define (domain forest)
  (:types
     ;;;;;;;top level grid;;;;;;;;
     location 
     problem

     ;;;;;;;subproblem;;;;;;;
     sub-location 
     log-location
     package
     truck
     plane
     city
     block
  )
  (:constants
	 x1 x2 
         y1 y2 - location

     ;;;;;;;sub-problem types;;;;;
     grid logistics blocksworld - problem

     ;;;;;;;blocksworld subproblem;;;
     a b c - block

     ;;;;;;;logistics subproblem;;;
     t1 t2 - truck
     a1 - plane
     p1 - package
     l11 l12 l21 l22 - log-location
     c1 c2 - city
  )
  (:predicates 
    ;;grid coordinates
   (at ?loc - location)
   (succ-loc ?loc1 ?loc2 - location)

    ;;subproblem at grid point is solved
   (solved ?loc - location)

   ;;subproblem type at each grid point
   (problem-at ?loc - location ?p - problem)

   ;;the subproblem at grid point can be solved
   (enabled ?loc - location) 
    ;;used to link subproblems 
   (enables ?loc ?loc1 - location)

   ;;;;;grid navigation subproblem;;;;;;;;;;;;;
   (s-at-x ?loc - sub-location ?l - location)
   (s-at-y ?loc - sub-location ?l - location)
   (s-succ-loc ?loc1 ?loc2 - sub-location)
   (s-goal-x ?loc - sub-location ?l - location)
   (s-goal-y ?loc - sub-location ?l - location)


   ;;;;;logistics subproblem;;;;;;;;;;;;;;;;;;;
   (s-pkg-at ?p - package ?loc - log-location ?l - location)
   (s-truck-at ?t - truck ?loc - log-location ?l - location)
   (s-plane-at ?t - plane ?loc - log-location ?l - location)
   (s-city-loc ?loc - log-location ?c - city)
   (s-airport-loc ?loc - log-location)
   (s-pkg-in-t ?p - package ?t - truck ?l - location)
   (s-pkg-in-a ?p - package ?a - plane ?l - location)

   ;;;;;blocksworld subproblem;;;;;;;;;;;;;;;;;
   (s-on ?b1 ?b2 - block ?l - location)
   (s-on-table ?b - block ?l - location)
   (s-clear ?b - block ?l - location)
   (s-empty ?l - location)
   (s-holding ?b - block ?l - location)

  )

  ;;;;;top-level grid navigation actions;;;;;;;

  (:action up-x
   :effect (and
	(when (and (at x1) (at y1) (solved x1)) 
              (and (not (at x1)) (at x2)))
	(when (and (at x1) (at y2) (solved x1)) 
              (and (not (at x1)) (at x2)))
  ))

  (:action up-y
   :effect (and
	(when (and (at x1) (at y1) (solved y1)) 
              (and (not (at y1)) (at y2)))
	(when (and (at x2) (at y1) (solved y1)) 
              (and (not (at y1)) (at y2)))
 ))

  (:action down-x
   :effect (and
	(when (and (at x2) (at y1) (solved x2)) 
              (and (not (at x2)) (at x1)))
	(when (and (at x2) (at y2) (solved x2)) 
              (and (not (at x2)) (at x1)))
  ))

  (:action down-y
   :effect (and
	(when (and (at x1) (at y2) (solved y2)) 
              (and (not (at y2)) (at y1)))
	(when (and (at x2) (at y2) (solved y2)) 
              (and (not (at y2)) (at y1)))
 ))


 (:action enable_subproblem
  :parameters (?loc ?loc1 - location)
  ;:precondition (and)
  :effect (when (and (solved ?loc) (enables ?loc ?loc1)) (enabled ?loc1))
 )

  ;;;;;grid navigation sub-problem;;;;;;;;;;;


  (:action s-up-x
   :parameters (?sloc1x ?sloc1y ?sloc2x - sub-location
                ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc) 
			(s-succ-loc ?sloc1x ?sloc2x)	
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-x ?sloc2x ?loc) (not (s-at-x ?sloc1x ?loc))))
  )

  (:action s-up-y
   :parameters (?sloc1x ?sloc1y ?sloc2y - sub-location
                ?loc - location)
   :precondition (and) 
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
			(s-succ-loc ?sloc1y ?sloc2y)
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-y ?sloc2y ?loc) (not (s-at-y ?sloc1y ?loc))))
  )

  (:action s-down-x
   :parameters (?sloc1x ?sloc1y ?sloc2x - sub-location
                ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
			 (s-succ-loc ?sloc2x ?sloc1x)
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-x ?sloc2x ?loc) (not (s-at-x ?sloc1x ?loc))))
  )

  (:action s-down-y
   :parameters (?sloc1x ?sloc1y ?sloc2y - sub-location
                ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)
			(s-succ-loc ?sloc2y ?sloc1y)
		      (s-at-x ?sloc1x ?loc) (s-at-y ?sloc1y ?loc))
		(and (s-at-y ?sloc2y ?loc) (not (s-at-y ?sloc1y ?loc))))
  )

  (:action solve-grid
   :parameters (?slocx ?slocy - sub-location
	        ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc grid) (at ?loc)    
		      (s-at-x ?slocx ?loc) (s-at-y ?slocy ?loc)
                      (s-goal-x ?slocx ?loc) (s-goal-y ?slocy ?loc)
                 )
		(solved ?loc) 
           )
  )


  ;;;;;logistics sub-problem;;;;;;;;;;;


  (:action load-truck
   :parameters (?t - truck ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
    	 	      (s-truck-at ?t ?loc ?l) (s-pkg-at ?p ?loc ?l))
                 (and (s-pkg-in-t ?p ?t ?l) (not (s-pkg-at ?p ?loc ?l))))
  )

  (:action load-plane
   :parameters (?a - plane ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-plane-at ?a ?loc ?l) (s-pkg-at ?p ?loc ?l))
		(and (s-pkg-in-a ?p ?a ?l) (not (s-pkg-at ?p ?loc ?l))))
  )

  (:action unload-truck
   :parameters (?t - truck ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-truck-at ?t ?loc ?l) (s-pkg-in-t ?p ?t ?l))
		(and (s-pkg-at ?p ?loc ?l) (not (s-pkg-in-t ?p ?t ?l))))
  )

  (:action unload-plane
   :parameters (?a - plane ?p - package ?loc - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-plane-at ?a ?loc ?l) (s-pkg-in-a ?p ?a ?l))
		(and (s-pkg-at ?p ?loc ?l) (not (s-pkg-in-a ?p ?a ?l))))
  )

  (:action move-truck
   :parameters (?t - truck ?loc1 ?loc2 - log-location ?c - city ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-truck-at ?t ?loc1 ?l) (s-city-loc ?loc1 ?c)
                      (s-city-loc ?loc2 ?c) (not (= ?loc1 ?loc2))
                 )
		(and (not (s-truck-at ?t ?loc1 ?l)) (s-truck-at ?t ?loc2 ?l)))
  ) 

  (:action move-plane
   :parameters (?a - plane ?loc1 ?loc2 - log-location ?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
		      (s-plane-at ?a ?loc1 ?l) (s-airport-loc ?loc1)
                      (s-airport-loc ?loc2) (not (= ?loc1 ?loc2))
                 )
		(and (not (s-plane-at ?a ?loc1 ?l)) (s-plane-at ?a ?loc2 ?l)))
  ) 

  (:action solve-logistics
   :parameters (?l - location)
   :precondition (and)
   :effect (when (and (enabled ?l) (problem-at ?l logistics) (at ?l)
                      (s-pkg-at p1 l22 ?l)
                 )
	        (solved ?l) 
           )
  )


  ;;;;;blocksworld sub-problem;;;;;;;;;;;
  (:action stack
   :parameters (?b1 ?b2 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
		       (s-holding ?b1 ?loc) (s-clear ?b2 ?loc))
	 	(and (s-on ?b1 ?b2 ?loc) (not (s-holding ?b1 ?loc)) 
                     (not (s-clear ?b2 ?loc)) (s-clear ?b1 ?loc) (s-empty ?loc)))
  )

  (:action putdown
   :parameters (?b1 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-holding ?b1 ?loc))
		(and (s-on-table ?b1 ?loc) (not (s-holding ?b1 ?loc)) 
                     (s-clear ?b1 ?loc) (s-empty ?loc)))
  )

  (:action unstack
   :parameters (?b1 ?b2 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-empty ?loc) (s-clear ?b1 ?loc) (s-on ?b1 ?b2 ?loc))
		(and (not (s-on ?b1 ?b2 ?loc)) (s-holding ?b1 ?loc) (s-clear ?b2 ?loc)
                     (not (s-clear ?b1 ?loc)) (not (s-empty ?loc))))
  )

  (:action pickup
   :parameters (?b1 - block ?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-empty ?loc) (s-clear ?b1 ?loc)
                 )
		(and (not (s-on-table ?b1 ?loc)) (s-holding ?b1 ?loc) 
                     (not (s-clear ?b1 ?loc)) (not (s-empty ?loc))))
  )

  (:action solve-blocksworld
   :parameters (?loc - location)
   :precondition (and)
   :effect (when (and (enabled ?loc) (problem-at ?loc blocksworld) (at ?loc)
                      (s-on a b ?loc) (s-on b c ?loc)
                 )
		(solved ?loc) 
           )
  )

)


(define (problem p1)
 (:domain forest)
 (:objects sx1 sx2 
           sy1 sy2 - sub-location

 )
 (:init 
	;;;;;;;;top level grid;;;;;;;
	(oneof (at x1) (at x2))
	(oneof (at y1) (at y2))
	
;	(at-x x1)
;	(at y1)

;	(solved y1)
;	(solved y2)

	(succ-loc x1 x2)
	(succ-loc y1 y2)


	;;;;;;;;enablings;;;;;;;;;;;
        (enabled y1)
        (enabled y2)
        (enabled x1)
        (enabled x2)
	(enables y1 y2)
        (enables x2 x1)
	(enables y1 x2)


	;;;;;;;;subproblems;;;;;;;;;;
;       (oneof 
;	(problem-at x1 grid) 
        ;(problem-at x1 logistics) 
        (problem-at x1 blocksworld)
;       )
;       (oneof 
;	(problem-at x2 grid) 
        (problem-at x2 logistics) 
        ;(problem-at x2 blocksworld)
;       )
;       (oneof 
;	(problem-at y1 grid) 
        ;(problem-at y1 logistics) 
        (problem-at y1 blocksworld)
;       )
;      (oneof
;	(problem-at y2 grid) 
        (problem-at y2 logistics) 
        ;(problem-at y2 blocksworld)
;      )


	;;;;;;;;grid sub-problem;;;;;;
             
	(s-at-x sx2 x1) 
	(s-at-y sy2 x1)
	(s-goal-x sx1 x1) 
	(s-goal-y sy1 x1)

	(s-at-x sx2 x2) 
	(s-at-y sy2 x2)
	(s-goal-x sx1 x2) 
	(s-goal-y sy1 x2)

	(s-at-x sx2 y1) 
	(s-at-y sy2 y1)
	(s-goal-x sx1 y1) 
	(s-goal-y sy1 y1)


	(s-at-x sx2 y2) 
	(s-at-y sy2 y2)
	(s-goal-x sx1 y2) 
	(s-goal-y sy1 y2)
	
	;;;all grid problems
	(s-succ-loc sx1 sx2)
	(s-succ-loc sy1 sy2)

	;;;;;;logistics sub-problem;;
        (s-city-loc l11 c1) (s-city-loc l12 c1)
        (s-city-loc l21 c2) (s-city-loc l22 c2)
        (s-airport-loc l11) (s-airport-loc l21)


	(s-pkg-at p1 l12 x1) (s-truck-at t1 l12 x1)
        (s-truck-at t2 l21 x1) (s-plane-at a1 l11 x1)

	(s-pkg-at p1 l12 x2) (s-truck-at t1 l12 x2)
        (s-truck-at t2 l21 x2) (s-plane-at a1 l11 x2)

	(s-pkg-at p1 l12 y1) (s-truck-at t1 l12 y1)
        (s-truck-at t2 l21 y1) (s-plane-at a1 l11 y1)

	(s-pkg-at p1 l21 y2) (s-truck-at t1 l12 y2)
        (s-truck-at t2 l21 y2) (s-plane-at a1 l11 y2)
	
	;;;;;;blocks sub-problem;;;;;;
	(s-on c a x1) (s-on-table a x1) (s-on-table b x1) 
        (s-clear b x1) (s-clear c x1) (s-empty x1)

	(s-on c a x2) (s-on-table a x2) (s-on-table b x2) 
        (s-clear b x2) (s-clear c x2) (s-empty x2)

	(s-on c a y1) (s-on-table a y1) (s-on-table b y1) 
        (s-clear b y1) (s-clear c y1) (s-empty y1)

	(s-on c a y2) (s-on-table a y2) (s-on-table b y2) 
        (s-clear b y2) (s-clear c y2) (s-empty y2)


 )
 (:goal 
	(and (at x2) (at y2) 

        )
 )
)